/* 2013-09-05: File added and changed by Sony Corporation */
/* icd_board.c
 *
 * Copyright 2012 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#if defined(CONFIG_ICD)

#include "reg.h"
#include "icx_io.h"
#include "icx_gpio.h"
#include "icd_board_type.h"

#define INPUT_OPEN       0x44444444
#define INPUT_PULLUP     0x55555555
#define INPUT_PULLDOWN   0x77777777
#define INPUT_SCHMITT_OPEN	0xeeeeeeee

#define CHG_DRIVE1_SD_SDI0_MASK 0xf0003c00 // SD, SDI0
#define CHG_DRIVE1_SD_CLK4_CMD4_SDI0_CLK4_CMD8	0x00002000 // SD:CLK 4mA, CMD 4mA, SDIO:CLK 4mA, CMD 8mA
#define CHG_DRIVE1_CLK4_CMD8    0x80002000 // CLK 4mA, CMD 8mA
#define CHG_DRIVE1_CLK8_CMD8    0xa0002800 // CLK 8mA, CMD 8mA

/* board id GPIOs */
enum {
    BID_PIN_1 = 0,
    BID_PIN_2,
    BID_PIN_MAX
};

struct bid_pin_info {
    u32 gpio;
    u32 pull_reg;
    u32 pull_mask;
};

static struct bid_pin_info bid_pin[BID_PIN_MAX] = {
    { GPIO_P015, CHG_PULL1, 0xf0000000 },   // BID_PIN_1
    { GPIO_P019, CHG_PULL2, 0x0000f000 },   // BID_PIN_2
};

static int s_board_id1 = 0;
static int s_board_id2 = 0;

#define PIN_CHECK(pin)  { if(pin < BID_PIN_1 || BID_PIN_MAX <= pin) return -1; }

/*-----------------------------------------------------------------------------
 * Function	: get_gpio_with_pull
 * Abstruct : set pull up/down then read gpio value
 * Input    : pin	: gpio pin
 *			: pull	: pull up/down/none
 * Return   : gpio value
 *----------------------------------------------------------------------------*/
static int set_gpio_pull(int pin, int pull)
{
    u32 reg;
    u32 mask;

    PIN_CHECK(pin);

    reg  = bid_pin[pin].pull_reg;
    mask = bid_pin[pin].pull_mask;

    outl((inl(reg) & ~mask) | (pull & mask), reg);
    return 0;
}

/*-----------------------------------------------------------------------------
 * Function	: get_gpio_with_pull
 * Abstruct : set pull up/down then read gpio value
 * Input    : pin	: gpio pin
 *			: pull	: pull up/down/none
 * Return   : gpio value
 *----------------------------------------------------------------------------*/
static int get_gpio_with_pull(int pin, int pull)
{
    PIN_CHECK(pin);

    set_gpio_pull(pin, pull);
    udelay(100);
    return gpio_get_value(bid_pin[pin].gpio);
}

/*-----------------------------------------------------------------------------
 * Function	: get_gpio_with_pull
 * Abstruct : set pull up/down then read gpio value
 * Input    : pin	: gpio pin
 *			: pull	: pull up/down/none
 * Return   : gpio value
 *----------------------------------------------------------------------------*/
static int get_board_id(int pin)
{
    int bid;
    PIN_CHECK(pin);

    // direction input
    gpio_set_direction(bid_pin[pin].gpio, GPIO_IN);

    // board id detect
    if(get_gpio_with_pull(pin, INPUT_OPEN)) {
        // PU(1k or 470k)
        if(!get_gpio_with_pull(pin, INPUT_PULLDOWN)) {
            bid = 0; // PU(470k)
        }
        else {
            bid = 1; // PU(1k)
        }
    }
    else {
        // PD
        if(get_gpio_with_pull(pin, INPUT_PULLUP)) {
            bid = 2;// PD(470k)
        }
        else {
            bid = 3;// PD(1k)
        }
    }

    // set gpio pin open
    set_gpio_pull(pin, INPUT_SCHMITT_OPEN);

    return bid;
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_config
 * Abstruct : config board by board id
 * Return   : none
 *----------------------------------------------------------------------------*/
static void board_config(int id1, int id2)
{
    u32 reg = inl(CHG_DRIVE1);

    reg &= ~CHG_DRIVE1_SD_SDI0_MASK; // clear bits

    if(icd_board_type_is_set()) {
        /* set */
#if defined(CONFIG_ICD_0425)
        reg |= CHG_DRIVE1_SD_CLK4_CMD4_SDI0_CLK4_CMD8;
#else
        reg |= CHG_DRIVE1_CLK4_CMD8;
#endif
        //printf("[drv] set 0x%08x\n", reg);
    }
    else {
        /* BB */
        reg |= CHG_DRIVE1_CLK8_CMD8;
        //printf("[drv] bb 0x%08x\n", reg);
    }

    outl(reg, CHG_DRIVE1);
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_id_init
 * Abstruct : initialize and detect board id
 * Return   : none
 *----------------------------------------------------------------------------*/
void icd_board_type_init(void)
{
    // get board ids
    s_board_id1 = get_board_id(BID_PIN_1);
    s_board_id2 = get_board_id(BID_PIN_2);

    // print
    printf("\n\nICD BOARD: ID1=%d,ID2=%d (%s)", s_board_id1, s_board_id2, icd_board_type_get_string());

    // board depend setting (eg. drive strength)
    board_config(s_board_id1, s_board_id2);
}

/*-----------------------------------------------------------------------------
 * Function	: icd_get_board_id
 * Abstruct : get board id
 * Return   : board id (refer icd hw spec)
 *----------------------------------------------------------------------------*/
int icd_board_type_get_id1(void) { return s_board_id1; }
int icd_board_type_get_id2(void) { return s_board_id2; }

/*-----------------------------------------------------------------------------
 * Function	: icd_get_board_id_string
 * Abstruct : board id -> board string
 * Input    : id1	: board id1
 *            id2	: board id2
 * Return   : board id string
 *----------------------------------------------------------------------------*/
const char* icd_board_type_get_string(void)
{
    if(s_board_id1 == 0 && s_board_id2 == 0) return "BB";
    if(s_board_id1 == 1 && s_board_id2 == 0) return "SET DEBUG";
    if(s_board_id1 == 2 && s_board_id2 == 0) return "SET";
    if(s_board_id1 == 3 && s_board_id2 == 0) return "SET CHECKER";

    return "unknown";
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_is_set
 * Abstruct : board is set?
 * Return   : 1=yes set, 0=no not set
 *----------------------------------------------------------------------------*/
int icd_board_type_is_set(void)
{
    if((s_board_id1 == 1 && s_board_id2 == 0) ||
       (s_board_id1 == 2 && s_board_id2 == 0) ||
       (s_board_id1 == 3 && s_board_id2 == 0)) {
        return 1;
    }
    else {
        return 0;
    }
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_is_no_uart
 * Abstruct : disable uart?
 * Return   : 1=yes disable, 0=no enable
 *----------------------------------------------------------------------------*/
int icd_board_type_is_no_uart(void)
{
    if((s_board_id1 == 0 && s_board_id2 == 0) ||
       (s_board_id1 == 2 && s_board_id2 == 0)) {
        return 1;
    }
    else {
        return 0;
    }
}

#endif

